import sys
from loguru import logger
from typing import Optional


def set_logger(
    format: Optional[str] = None,
    level: str = "INFO",
    colorize: bool = True,
    diagnose: bool = True,
    catch: bool = True,
    use_file: bool = False,
    file_path: Optional[str] = "./log/log.log",
    file_format: Optional[str] = None,
    file_rotation: str = "00:00",
    file_retention: str = "3 days",
    file_diagnose: bool = True,
    file_catch: bool = True,
    file_level: str = "INFO",
) -> logger:
    """loguru的二次封装。
    强调了开箱即用感。
    因为很不满意loguru的默认logger,所以自己再造了一个呢。
    大概就是那种你行你上的感觉吧。(笑)
    抱着这样的想法写出了这个函数。
    嘛,不过那种事情怎么样都好啦。
    当然,头发最后由工作人员好好的打扫掉了。

    Args:
        format (Optional[str], optional): loguru的format格式,默认为:`"<g>{time:YYYY-MM-DD HH:mm:ss}</g> | <m>{module}:{function}</m> | <lvl>{level}</lvl> | <lvl>{message}</lvl>"`. Defaults to None.
        level (str, optional): loguru的level级别. Defaults to "INFO".
        colorize (bool), optional): 是否上色(日志会有颜色). Defaults to True.
        diagnose (bool, optional): 是否跟踪诊断原因. Defaults to True.
        catch (bool, optional): logger是否可以catch,使用`@logger.catch`捕获报错信息. Defaults to True.
        use_file (bool, optional): 是否将日志输出到文件. Defaults to False.
        file_path (Optional[str], optional): 文件的路径. Defaults to "./log/log.log".
        file_format (Optional[str], optional): 日志文件使用的格式,要求符合loguru的format,默认同上. Defaults to None.
        file_rotation (str, optional): 什么时候更换为新日志记录. Defaults to "00:00".
        file_retention (str, optional): 日志保留的时间. Defaults to "3 days".
        file_diagnose (bool, optional): 是否跟踪诊断原因. Defaults to True.
        file_catch (bool, optional): logger是否可以catch然后输出至日志,使用方法同上. Defaults to True.
        file_level (str, optional): 日志文件的level. Defaults to "INFO".

    Returns:
        logger: 二次封装后的logger,是自己日常常用的风格
    """
    logger.remove()

    logger.add(
        sys.stderr,
        format=format
        if format
        else "<g>{time:YYYY-MM-DD HH:mm:ss}</g> | <m>{module}:{function}</m> | <lvl>{level}</lvl> | <lvl>{message}</lvl>",
        level=level,
        diagnose=diagnose,
        catch=catch,
        colorize=colorize,
    )
    if use_file:
        logger.add(
            file_path,
            format=file_format
            if file_format
            else "<g>{time:YYYY-MM-DD HH:mm:ss}</g> | <m>{module}:{function}</m> | <lvl>{level}</lvl> | <lvl>{message}</lvl>",
            rotation=file_rotation,
            retention=file_retention,
            diagnose=file_diagnose,
            catch=file_catch,
            level=file_level,
        )

    return logger


logger = set_logger()



from .uihook import hook_manager
from .uilog import set_logger, logger
from .uio import sync_uio, async_uio
from .uitry import retry
from .uiutils import sync_to_async
from .uiclient import sync_uiclient, async_uiclient


__all__ = [
    "sync_with_hook",
    "async_with_hook",
    "set_logger",
    "logger",
    "sync_uio",
    "async_uio",
    "sync_uiclient",
    "async_uiclient",
    "retry",
    "sync_to_async",
]
__version__ = "0.0.6"

sync_with_hook = hook_manager().sync_with_hook
async_with_hook = hook_manager().async_with_hook
sync_uio = sync_uio()
async_uio = async_uio()



from typing import Any, Callable

from .uilog import logger


class hook_manager:
    def __init__(self) -> None:
        """预设解析函数与终止函数,以免报错Not Found"""
        self.parser_func = None
        self.ending_func = None

    def parser(self, func: Callable) -> Callable:
        """记录解析函数,用作装饰器时记得接受一个字典用作在解析,主以及终止函数中相互传递

        Args:
            func (Callable): 用户编写的解析函数

        Returns:
            Callable: 返回解析函数
        """
        self.parser_func = func
        return func

    def asnyc_runner(self, func: Callable) -> Callable:
        """聪明的羽衣可以异步的将main函数切换为run函数

        Args:
            func (Callable): 用户编写的main函数

        Returns:
            Callable: run函数
        """

        async def run(*arg, **kw) -> None:
            """主函数,检测是否定义了解析函数与终止函数,并且顺序执行

            Returns:
                dict[str, Any]: 记录三个函数各自的返回,以及在函数中传递的字典states
            """
            self.states = {}
            if self.parser_func:
                try:
                    parser_res = await self.parser_func(self.states)
                except Exception as e:
                    logger.warning(f"执行预处理函数时出现错误: {e}")
                    self.states["parser_res"] = e
                else:
                    self.states["parser_res"] = parser_res

            try:
                main_res = await self.func(self.states, *arg, **kw)
            except Exception as e:
                logger.warning(f"执行主函数时出现错误: {e}")
                self.states["parser_res"] = e
            else:
                self.states["parser_res"] = main_res

            if self.ending_func:
                try:
                    ed_res = await self.ending_func(self.states)
                except Exception as e:
                    logger.warning(f"执行终止函数时出现错误: {e}")
                    self.states["parser_res"] = e
                else:
                    self.states["parser_res"] = ed_res

        self.func = func
        return run

    def sync_runner(self, func: Callable) -> Callable:
        """聪明的羽衣非常想异步但是只能同步的将main函数切换为run函数

        Args:
            func (Callable): 用户编写的main函数

        Returns:
            Callable: run函数
        """

        def run(*arg, **kw) -> dict[str, Any]:
            """主函数,检测是否定义了解析函数与终止函数,并且顺序执行

            Returns:
                dict[str, Any]: 记录三个函数各自的返回,以及在函数中传递的字典states
            """
            self.states = {}
            if self.parser_func:
                try:
                    parser_res = self.parser_func(self.states)
                except Exception as e:
                    logger.warning(f"执行预处理函数时出现错误: {e}")
                    self.states["parser_res"] = e
                else:
                    self.states["parser_res"] = parser_res

            try:
                main_res = self.func(self.states, *arg, **kw)
            except Exception as e:
                logger.warning(f"执行主函数时出现错误: {e}")
                self.states["parser_res"] = e
            else:
                self.states["parser_res"] = main_res

            if self.ending_func:
                try:
                    ed_res = self.ending_func(self.states)
                except Exception as e:
                    logger.warning(f"执行终止函数时出现错误: {e}")
                    self.states["parser_res"] = e
                else:
                    self.states["parser_res"] = ed_res

        self.func = func
        return run

    def end_func(self, func: Callable) -> Callable:
        """记录终止函数,用作装饰器时记得接受一个字典用作在解析,主以及终止函数中相互传递

        Args:
            func (Callable): 用户编写的终止函数

        Returns:
            Callable: 返回终止函数
        """
        self.ending_func = func
        return func

    @staticmethod
    def sync_with_hook() -> Callable:
        """唯一且应该调用的接口,用于注册一个hook函数,在调用时会传递一个字典`hook_manager.states`用于几个函数间的交互具体示例如下
        ```
        @hook_manager.sync_with_hook()
        def test(session):
            # 在调用argp后会调用test
            # session = {'start'='first'}
            session['main'] = 'second'
        @test.arg_parser
        def argp(session):
            # 这个函数会最先被调用
            session['start'] = 'first'
        @test.ending
        def endg(session):
            # 这个函数会在完成test后调用
            # 在调用argp后会调用test
            # session = {'start'='first','main':'second'}
            session['last'] = 'third'
        ```

        Returns:
            Callable: 返回deco函数,请注意！！！在如上示例的运行后,`test`函数实际上是`hook_manager.run`函数
        """

        def deco(func: Callable) -> Callable:
            """真正的去注册一个hook函数

            Args:
                func (Callable): 用户编写的函数

            Returns:
                Callable: hook_manager.run
            """
            hk = hook_manager()
            func = hk.sync_runner(func)
            func.args_parser = hk.parser
            func.ending = hk.end_func
            return func

        return deco

    @staticmethod
    def async_with_hook() -> Callable:
        """唯一且应该调用的接口,用于注册一个hook函数,在调用时会传递一个字典`hook_manager.states`用于几个函数间的交互具体示例如下
        ```
        @hook_manager.async_with_hook()
        def test(session):
            # 在调用argp后会调用test
            # session = {'start'='first'}
            session['main'] = 'second'
        @test.arg_parser
        def argp(session):
            # 这个函数会最先被调用
            session['start'] = 'first'
        @test.ending
        def endg(session):
            # 这个函数会在完成test后调用
            # 在调用argp后会调用test
            # session = {'start'='first','main':'second'}
            session['last'] = 'third'
        ```

        Returns:
            Callable: 返回deco函数,请注意！！！在如上示例的运行后,`test`函数实际上是`hook_manager.run`函数
        """

        def deco(func: Callable) -> Callable:
            """真正的去注册一个hook函数

            Args:
                func (Callable): 用户编写的函数

            Returns:
                Callable: hook_manager.run
            """
            hk = hook_manager()
            func = hk.asnyc_runner(func)
            func.args_parser = hk.parser
            func.ending = hk.end_func
            return func

        return deco



import asyncio
from typing import Awaitable, Callable
from functools import wraps, partial


def sync_to_async(func: Callable) -> Awaitable:
    """将同步函数变为异步,使用方法:
    ```
    @sync_to_async
    def abc():
        print("success")

    async def cde():
        await abc()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(asyncio.wait([cde()]))
    >>> success
    ```

    Args:
        func (Callable): 需要被装饰函数

    Returns:
        Awaitable: 异步函数
    """

    @wraps(func)
    async def run(*args, loop=None, executor=None, **kwargs):
        if loop is None:
            loop = asyncio.get_event_loop()
        pfunc = partial(func, *args, **kwargs)
        return await loop.run_in_executor(executor, pfunc)

    return run



from setuptools import setup

# TODO(shiying): 去找个愿意帮忙写README的苦工
# 2021/10/13 18:02 GMT+8 编辑: 奥！苦工竟是我自己！
with open("./README.md", "r", encoding="utf-8") as f:
    long_des = f.read()

setup(
    name="soraha_utils",
    version="0.0.6",
    author="shiying",
    author_email="839778960@qq.com",
    description="soraha_utils",
    long_description=long_des,
    long_description_content_type="text/markdown",
    url="https://www.github.com/LYshiying/soraha_utils",
    packages=["soraha_utils"],
    install_requires=[
        "httpx>=0.19.0",
        "ujson>=4.1.0",
        "loguru>=0.5.3",
        "requests>=2.26.0",
        "aiofiles>=0.7.0",
    ],
    classifiers=(
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ),
    python_requires=">=3.6",
)
